require "MiddleClass.lua"
require "AnAL.lua"
require "mud_sound.lua"

Character = class('Character')

function Character:initialize()
	
	-- Sprite animations
	self.animation = {}
	function self.animation.defineAnimation(name, sheet, framew, frameh, time, mode)
		self.animation[name] = function()
			self.sprite_src = love.graphics.newImage("sprites/" .. sheet)
			self.sprite = newAnimation(self.sprite_src, framew, frameh, time, 0)
			self.sprite:setMode(mode)
		end
	end
	self.animation.switchCurrent = function(anim)
		if self.animation[anim] ~= nil  and self.animation.current ~= anim then
			-- dont interrupt swing animations
			if (self.animation.current == "swing_right" or
			   self.animation.current == "swing_left") and
			   self.sprite:getCurrentFrame() < self.sprite:getSize() then 
				return
			end
				   
			self.animation.current = anim
			self.animation[anim]()
		end
	end
	
	-- Define our animations.
	self.animation.defineAnimation("standing_from_right", "standing_right.png", 64, 64, 0, "once")
	self.animation.defineAnimation("standing_from_left", "standing_left.png", 64, 64, 0, "once")
	self.animation.defineAnimation("right", "right_sprites.png", 64, 64, 0.1, "once")
	self.animation.defineAnimation("left", "left_sprites.png", 64, 64, 0.1, "once")
	self.animation.defineAnimation("swing_right", "swing_right.png", 128, 64, 0.1, "once")
	self.animation.defineAnimation("swing_left", "swing_left.png", 128, 64, 0.1, "once")
	
	-- Set initial animation to 'standing'
	self.animation.standing_from_right()
	self.animation.current = "standing"
	
	-- Other default attributes.
	self.x = 0
	self.y = 0
	self.jumping = false
	self.upvel = 300
	self.speed = 200
	self.fp = 100 -- flight power
	self.hp = 100 -- health points
	self.direction = "right"
	
	-- Load sound effects
	sound_load("jump", "audio/jump.wav")
	sound_load("land", "audio/land.wav")
	for i=1, 4 do
		sound_load("sword_miss_" .. i, "audio/miss" .. i .. ".wav")
	end
end

function Character:update(dt)
	if not(love.keyboard.isDown("right") and love.keyboard.isDown("left")) then
		if love.keyboard.isDown("right") then
			self.direction = "right"
			self.animation.switchCurrent(self.direction)
			self.x = math.floor(self.x + self.speed * dt)
		elseif love.keyboard.isDown("left") then
			self.direction = "left"
			self.animation.switchCurrent(self.direction)
			self.x = math.floor(self.x - self.speed * dt)
		else
			if self.y > 0 then
				self.animation.switchCurrent(self.direction)
			else
				self.animation.switchCurrent("standing_from_" .. self.direction)
			end
		end
	else
		self.animation.switchCurrent("standing_from_" .. self.direction)
	end
	
	if self.jumping then
		if self.y + self.upvel * dt < 400  then
			self.y = self.y + self.upvel * dt
			
			if self.fp > 0 then
				self.fp = self.fp - 30 * dt
			end
		end
		
		if love.keyboard.isDown("down") then
			self.upvel = self.upvel - 2000 * dt
		else
			self.upvel = self.upvel - 600 * dt
		end
		
		if self.y < 0 then
			self.jumping = false
			self.falling = false
			self.y = 0
			self.upvel = 300
			sound_play("land")
		end
	else
		if self.fp <100 then 
			self.fp = self.fp + 50 * dt
		end
	end
	

	self.sprite:update(dt)
end

function Character:keypressed(key,unicode)
	if key == 'z' then
		if self.fp > 0 then
			sound_play("jump")
		end
		
		if self.jumping  then
			if self.fp > 0 then
				self.upvel =  300
			end
		end
		
		self.jumping = true
	end
	
	if key == 'x' then
		self:doSwing()
	end
end

function Character:doSwing()
	if self.direction == "right" then 
		if ( c:checkHit(self.x+100, -self.y+412) or c:checkHit(self.x+50, -self.y+412)  ) then
			c.yvel = math.random(1,2) * 100 * -1
			c.xvel = math.random(1,2) * 100 
		end
	else
		if ( c:checkHit(self.x-30, -self.y+412) or c:checkHit(self.x, -self.y+412)  ) then
			c.yvel = math.random(1,2) * 100 * -1
			c.xvel = math.random(1,3) * 100 * -1
		end
	end
	

	self.animation.switchCurrent("swing_" .. self.direction)
	sound_play("sword_miss_" .. tostring(math.random(1,4)))
end

function Character:keyreleased(key,unicode)
	if key == 'z' then

	end
end

function Character:drawStatusBars()

	love.graphics.setColor(68,70,61)
	love.graphics.rectangle('fill', 20, 25, 50 * self.hp/100, 10)
	love.graphics.setColor(186,192,164)
	love.graphics.rectangle('fill', 100, 25, 50 * self.fp/100, 10)
	
	love.graphics.setColor(0,0,0)
	love.graphics.rectangle('line', 20, 25, 50, 10)
	love.graphics.rectangle('line', 100, 25, 50, 10)
	love.graphics.print("health", 20, 5 )
	love.graphics.print("flight", 100, 5 )
	
	love.graphics.setColor(255,255,255)
end

function Character:draw()
	-- Character shadow
	love.graphics.setColor(0,0,0,100)
	love.graphics.circle('fill', 263,470, 10 + self.y/15, 1280)
	love.graphics.setColor(255,255,255)
	if self.animation.current == "swing_left" then  -- workaround until I fix my shitty sprites.
			self.sprite:draw(220,412 - self.y)
	else
		self.sprite:draw(250,412 - self.y)
	end
	
	self:drawStatusBars()
end